
class RedisInfo:
    def __init__(self, host="localhost", port=6379, password=""):
        self.host = host
        self.port = port
        self.password = password


try:
    import redis
    use_redis = True
except ImportError:
    use_redis = False
    redis = None
class ProxyConfig:
    def __init__(self,redis_info=RedisInfo()):
        # Redis配置
        if use_redis:
            self.redis_client = redis.Redis(host=redis_info.host, port=redis_info.port, password=redis_info.password)
        else:
            self.redis_client = None
        self.threshold = 5#代理IP阈值
        self.check_url = "http://httpbin.org/ip"#验证地址
        self.timeout = 5#请求超时时间
        self.verify_proxy = True#是否验证代理IP的可用性开关
        self.ttl = 30#代理IP的存活时间
        self.api_url = ""
        self.verify_thread_num=10#验证代理IP的线程数
        self.get_time=3#获取代理IP的间隔时间(指的是从api获取代理IP的间隔时间)
        self.use_redis = use_redis  # 是否使用Redis模式，如果Redis不可用则默认为False
        self.redis_prefix="myapp"#redis前缀
        # 初始化Redis连接（如果启用）
        if self.use_redis:
            self.redis_client = self.redis_client
        else:
            self.redis_client = None
    def set_threshold(self, threshold=5):
        """
        设置代理IP阈值
        :param threshold: 代理IP阈值
        :return: None
        """
        self.threshold = threshold
    def set_get_time(self, get_time=3):
        """
        设置获取代理IP的间隔时间
        :param get_time: 获取代理IP的间隔时间
        :return: None
        """
        self.get_time = get_time
    def set_redis_prefix(self, redis_prefix="myapp"):
        """
        设置redis前缀
        :param redis_prefix: redis前缀
        :return: None
        """
        self.redis_prefix = redis_prefix
    def set_check_url(self, check_url="http://httpbin.org/ip"):
        """
        设置验证地址
        :param check_url: 验证地址
        :return: None
        """
        self.check_url = check_url
    def set_timeout(self, timeout=5):
        """
        设置请求超时时间
        :param timeout: 请求超时时间
        :return: None
        """
        self.timeout = timeout
    def set_verify_proxy(self, verify_proxy=True):
        """
        设置是否验证代理IP的开关
        :param verify_proxy: 是否验证代理IP的开关
        :return: None
        """
        self.verify_proxy = verify_proxy
    def set_ttl(self, ttl=30):
        """
        设置代理IP的存活时间
        :param ttl: 代理IP的存活时间
        :return: None
        """
        self.ttl = ttl
    def set_api_url(self, api_url="http://your-proxy-api.com"):
        """
        设置代理API地址
        :param api_url: 代理API地址
        :return: None
        """
        self.api_url = api_url
    def set_verify_thread_num(self, verify_thread_num=10):
        """
        设置验证代理IP的线程数
        :param verify_thread_num: 验证代理IP的线程数
        :return: None
        """
        self.verify_thread_num = verify_thread_num


import hashlib
import sys
import subprocess
import platform
import os
import re  # 补充缺失的正则模块导入
from typing import Dict, List, Optional


def execute_cmd(cmd: str, encoding: str = "utf-8", need_sudo: bool = False) -> str:
    """
    通用命令执行函数：适配Windows/Linux编码，自动处理sudo权限
    :param cmd: 执行的命令
    :param encoding: 编码（Windows=cp936/gbk，Linux=utf-8）
    :param need_sudo: 是否需要sudo（仅Linux）
    :return: 清洗后的命令输出
    """
    if need_sudo and sys.platform == "linux" and os.geteuid() != 0:
        cmd = f"sudo {cmd}"

    try:
        output = subprocess.check_output(
            cmd, shell=True, encoding=encoding, errors="ignore",
            stderr=subprocess.DEVNULL
        ).strip()
        # 清洗空行和多余空格
        return "\n".join([line.strip() for line in output.split("\n") if line.strip()])
    except Exception:
        return ""


def get_linux_disk_device() -> str:
    """自动识别Linux系统盘设备名（兼容sda/vda/xvda等）"""
    # 方式1：从/etc/mtab获取根分区设备
    mtab_cmd = 'cat /etc/mtab | grep " / " | awk \'{print $1}\''
    root_dev = execute_cmd(mtab_cmd)
    if root_dev:
        # 提取设备名（如/dev/sda1 → sda，/dev/vda2 → vda）
        dev_name = re.sub(r'\/dev\/(\w+)\d+', r'\1', root_dev)
        return dev_name if dev_name else "sda"
    # 方式2：兜底返回sda
    return "sda"


def get_hardware_identifier() -> Dict[str, Optional[str]]:
    """
    跨Windows/Linux精准获取硬件标识（优化Linux兼容性，适配CentOS 9/虚拟化环境）
    """
    result = {
        "os": platform.system(),
        "motherboard_serial": None,  # 主板序列号
        "bios_serial": None,  # BIOS序列号
        "cpu_serial": None,  # CPU序列号
        "disk_serial": None,  # 硬盘序列号（硬件/卷序列号兜底）
        "gpu_info": None,  # 显卡信息（所有显卡|分隔）
        "cpu_info": None,  # CPU型号
        "system_version": None,  # 系统版本
        "system_uuid": None,  # 系统UUID
        "physical_gpu": None,  # 物理显卡
        #"device_unique_id": None  # 设备唯一标识（兜底方案）
    }

    # ========== 1. Windows系统（保留精准逻辑） ==========
    if sys.platform == "win32":
        # 1.1 主板序列号
        mb_cmd = 'wmic baseboard get serialnumber'
        mb_output = execute_cmd(mb_cmd, encoding="cp936")
        result["motherboard_serial"] = mb_output.split("\n")[1].strip() if len(mb_output.split("\n")) >= 2 else "未知"

        # 1.2 BIOS序列号
        bios_cmd = 'wmic bios get serialnumber'
        bios_output = execute_cmd(bios_cmd, encoding="cp936")
        result["bios_serial"] = bios_output.split("\n")[1].strip() if len(
            bios_output.split("\n")) >= 2 else "SYSTEM SERIAL NUMBER"

        # 1.3 CPU序列号
        cpu_serial_cmd = 'wmic cpu get processorid'
        cpu_serial_output = execute_cmd(cpu_serial_cmd, encoding="cp936")
        result["cpu_serial"] = cpu_serial_output.split("\n")[1].strip() if len(
            cpu_serial_output.split("\n")) >= 2 else "未知"

        # 1.4 CPU型号
        cpu_model_cmd = 'wmic cpu get name'
        cpu_model_output = execute_cmd(cpu_model_cmd, encoding="cp936")
        result["cpu_info"] = cpu_model_output.split("\n")[1].strip() if len(
            cpu_model_output.split("\n")) >= 2 else platform.processor()

        # 1.5 硬盘序列号
        disk_cmd = 'wmic diskdrive get serialnumber'
        disk_output = execute_cmd(disk_cmd, encoding="cp936")
        disk_lines = disk_output.split("\n")[1:]
        valid_disks = [line.strip() for line in disk_lines if line.strip() and line.strip() != "None"]
        result["disk_serial"] = " ".join(valid_disks) if valid_disks else "未知"

        # 1.6 显卡信息
        gpu_cmd = 'wmic path win32_VideoController get name'
        gpu_output = execute_cmd(gpu_cmd, encoding="cp936")
        gpu_lines = gpu_output.split("\n")[1:]
        valid_gpus = [line.strip().upper() for line in gpu_lines if line.strip() and line.strip() != "None"]
        result["gpu_info"] = "| ".join(valid_gpus) + "|" if valid_gpus else "未知"
        # 过滤物理显卡
        virtual_keywords = ["VIRTUAL", "REMOTE", "MICROSOFT BASIC", "GAMEVIEWER"]
        physical_gpus = [gpu for gpu in valid_gpus if not any(kw in gpu for kw in virtual_keywords)]
        result["physical_gpu"] = physical_gpus[0] if physical_gpus else "未知"

        # 1.7 系统版本
        try:
            os_ver = platform.version()
            os_release = platform.release()
            os_name = platform.system().upper()
            result["system_version"] = f"{os_name}_{os_release}_{os_ver}"
        except:
            result["system_version"] = "未知"

        # 1.8 系统UUID
        uuid_cmd = 'wmic csproduct get uuid'
        uuid_output = execute_cmd(uuid_cmd, encoding="cp936")
        result["system_uuid"] = uuid_output.split("\n")[1].strip().upper() if len(
            uuid_output.split("\n")) >= 2 else "未知"

    # ========== 2. Linux系统（深度兼容CentOS 9/虚拟化环境） ==========
    elif sys.platform == "linux":
        # 2.1 主板序列号（多层备用方案）
        mb_serial = "未知"
        # 方案1：dmidecode baseboard-serial
        mb_cmd1 = 'dmidecode -s baseboard-serial-number'
        mb_output1 = execute_cmd(mb_cmd1, need_sudo=True)
        if mb_output1 and mb_output1 not in ["Not Applicable", ""]:
            mb_serial = mb_output1
        # 方案2：dmidecode system-serial（备用）
        else:
            mb_cmd2 = 'dmidecode -s system-serial-number'
            mb_output2 = execute_cmd(mb_cmd2, need_sudo=True)
            if mb_output2 and mb_output2 not in ["Not Applicable", ""]:
                mb_serial = mb_output2
        result["motherboard_serial"] = mb_serial

        # 2.2 BIOS序列号
        bios_cmd = 'dmidecode -s bios-version'
        bios_output = execute_cmd(bios_cmd, need_sudo=True)
        result["bios_serial"] = bios_output if bios_output else "SYSTEM SERIAL NUMBER"

        # 2.3 CPU序列号（多层备用）
        cpu_serial = "未知"
        # 方案1：dmidecode processor-id
        cpu_cmd1 = 'dmidecode -s processor-id'
        cpu_output1 = execute_cmd(cpu_cmd1, need_sudo=True)
        if cpu_output1 and cpu_output1 not in ["Not Applicable", ""]:
            cpu_serial = cpu_output1
        # 方案2：cpuid（备用）
        else:
            cpu_cmd2 = 'cpuid | grep "Processor serial number" | cut -d: -f2'
            cpu_output2 = execute_cmd(cpu_cmd2).strip()
            if cpu_output2:
                cpu_serial = cpu_output2
        result["cpu_serial"] = cpu_serial

        # 2.4 CPU型号（精准解析）
        cpu_model = "未知"
        cpu_cmd = 'cat /proc/cpuinfo | grep -m1 "model name" | cut -d: -f2 | sed "s/^ *//g"'
        cpu_output = execute_cmd(cpu_cmd)
        if cpu_output:
            cpu_model = cpu_output
        result["cpu_info"] = cpu_model

        # 2.5 硬盘序列号（自动识别设备+卷序列号兜底）
        disk_serial = "未知"
        # 步骤1：自动识别系统盘设备名（sda/vda/xvda等）
        dev_name = get_linux_disk_device()  # 如sda/vda
        # 方案1：lsblk serial
        disk_cmd1 = f'lsblk -no serial /dev/{dev_name}'
        disk_output1 = execute_cmd(disk_cmd1)
        if disk_output1 and disk_output1 not in ["", "unknown"]:
            disk_serial = disk_output1
        # 方案2：hdparm（备用）
        else:
            disk_cmd2 = f'hdparm -I /dev/{dev_name} 2>/dev/null | grep "Serial Number" | cut -d: -f2'
            disk_output2 = execute_cmd(disk_cmd2).strip()
            if disk_output2:
                disk_serial = disk_output2
        # 方案3：blkid卷序列号（终极兜底）
        if disk_serial == "未知":
            disk_cmd3 = f'blkid /dev/{dev_name}1 2>/dev/null | grep "UUID" | cut -d= -f2 | sed "s/\"//g"'
            disk_output3 = execute_cmd(disk_cmd3).strip()
            if disk_output3:
                disk_serial = f"卷序列号-{disk_output3}"
        result["disk_serial"] = disk_serial

        # 2.6 显卡信息
        gpu_list = []
        # 方案1：lshw
        gpu_cmd1 = 'lshw -C display 2>/dev/null | grep -E "product:" | cut -d: -f2 | sed "s/^ *//g"'
        gpu_output1 = execute_cmd(gpu_cmd1, need_sudo=True)
        if gpu_output1:
            gpu_list = [line.strip().upper() for line in gpu_output1.split("\n") if line.strip()]
        # 方案2：nvidia-smi（NVIDIA备用）
        if not gpu_list:
            gpu_cmd2 = 'nvidia-smi --query-gpu=name --format=csv,noheader'
            gpu_output2 = execute_cmd(gpu_cmd2)
            if gpu_output2:
                gpu_list = [line.strip().upper() for line in gpu_output2.split("\n") if line.strip()]
        # 方案3：lspci（终极备用）
        if not gpu_list:
            gpu_cmd3 = 'lspci | grep -i "vga\|3d\|display" | cut -d: -f3 | sed "s/^ *//g"'
            gpu_output3 = execute_cmd(gpu_cmd3)
            if gpu_output3:
                gpu_list = [line.strip().upper() for line in gpu_output3.split("\n") if line.strip()]
        result["gpu_info"] = "| ".join(gpu_list) + "|" if gpu_list else "未知"
        result["physical_gpu"] = gpu_list[0] if gpu_list else "未知"

        # 2.7 系统版本（适配CentOS 9）
        sys_version = "未知"
        try:
            # 方案1：lsb_release（优先）
            distro_name = execute_cmd('lsb_release -si').upper()
            distro_ver = execute_cmd('lsb_release -sr')
            kernel_ver = platform.release()
            if distro_name and distro_ver:
                sys_version = f"LINUX_{distro_name}_{distro_ver}_KERNEL-{kernel_ver}"
            # 方案2：/etc/os-release（备用）
            else:
                os_name = execute_cmd('grep -E "^NAME=" /etc/os-release | cut -d= -f2 | sed "s/\"//g"').upper()
                os_ver = execute_cmd('grep -E "^VERSION_ID=" /etc/os-release | cut -d= -f2 | sed "s/\"//g"')
                if os_name and os_ver:
                    sys_version = f"LINUX_{os_name}_{os_ver}_KERNEL-{kernel_ver}"
                # 方案3：仅内核版本（兜底）
                else:
                    sys_version = f"LINUX_KERNEL-{kernel_ver}"
        except:
            sys_version = f"LINUX_{platform.release()}"
        result["system_version"] = sys_version

        # 2.8 系统UUID（稳定获取）
        uuid = "未知"
        # 方案1：dmi product_uuid
        uuid_cmd1 = 'cat /sys/class/dmi/id/product_uuid 2>/dev/null'
        uuid_output1 = execute_cmd(uuid_cmd1)
        if uuid_output1:
            uuid = uuid_output1.upper()
        # 方案2：blkid（备用）
        else:
            uuid_cmd2 = f'blkid /dev/{dev_name}1 2>/dev/null | grep "UUID" | cut -d= -f2 | sed "s/\"//g"'
            uuid_output2 = execute_cmd(uuid_cmd2)
            if uuid_output2:
                uuid = uuid_output2.upper()
        result["system_uuid"] = uuid

    # ========== 3. 其他系统 ==========
    else:
        for k in result.keys():
            result[k] = f"暂不支持{platform.system()}系统" if k != "os" else platform.system()
        return result

    # ========== 4. 设备唯一标识（终极兜底方案） ==========
    unique_parts = []
    # 优先：系统UUID（最稳定）
    if result["system_uuid"] != "未知":
        unique_parts.append(result["system_uuid"])
    # 其次：主板序列号
    if result["motherboard_serial"] != "未知":
        unique_parts.append(result["motherboard_serial"])
    # 最后：硬盘序列号（兜底）
    if result["disk_serial"] != "未知":
        unique_parts.append(result["disk_serial"].replace("卷序列号-", ""))

    # if unique_parts:
    #     result["device_unique_id"] = "_".join(unique_parts)
    # else:
    #     result["device_unique_id"] = "无法生成唯一标识"

    return result


def get_devices_md5(uppercase: bool = False) -> tuple[str, str]:

    """
    对硬件信息字典排序并生成MD5值
    :param hardware_info: get_hardware_identifier返回的字典
    :param uppercase: MD5结果是否大写（默认小写）
    :return: 32位MD5字符串
    """
    # 1. 按key升序排序，生成有序键值对列表
    hardware_info = get_hardware_identifier()
    sorted_items = sorted(hardware_info.items(), key=lambda x: x[0])

    # 2. 格式化：将None转为"未知"，拼接为"key:value|key:value"格式
    formatted_str = "|".join([f"{k}:{v if v is not None else '未知'}" for k, v in sorted_items])

    # 3. 生成MD5（统一使用utf-8编码，避免跨平台编码问题）
    md5_obj = hashlib.md5(formatted_str.encode("utf-8"))
    md5_result = md5_obj.hexdigest()

    # 4. 可选：转为大写
    if uppercase:
        md5_result = md5_result.upper()

    return md5_result, formatted_str  # 返回MD5和格式化字符串（便于调试）

# # ========== 测试调用 ==========
# if __name__ == "__main__":
#     hardware_info = get_hardware_identifier()
#     for key, value in hardware_info.items():
#         print(f"{key}: {value}")
#     hardware_md5, formatted_str = get_sorted_hardware_md5(hardware_info, uppercase=False)
#     print(f"设备唯一标识（MD5）：{hardware_md5}")
#     print(f"格式化字符串：{formatted_str}")
#     #设备唯一标识（MD5）：B42A4A9137D367C13D944EAD4B30ECE7


import logging
import re
import sys
import threading
import time

import requests
import concurrent.futures
from proxy_config import ProxyConfig



ip_pattern = r'(?:\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}:\d{1,5})'
# 全局代理池和锁，用于在所有实例间共享
_global_proxy_pool = []
_global_proxy_pool_lock = threading.Lock()
class ProxyClass:
    def __init__(self, proxy_config, logger=None):
        self.proxy_config = proxy_config
        self.logger = logger or logging.getLogger(__name__)
        # 全局代理池和锁，用于在所有实例间共享
        global _global_proxy_pool, _global_proxy_pool_lock
        self.proxy_pool = _global_proxy_pool
        self.proxy_pool_lock = _global_proxy_pool_lock
        if self.proxy_config.use_redis and self.proxy_config.redis_client:
            self.redis_client = self.proxy_config.redis_client
            self.redis_prefix = self.proxy_config.redis_prefix
            self.ttl = self.proxy_config.ttl
        else:
            self.redis_client = None
            self.redis_prefix = None
            self.ttl = self.proxy_config.ttl
        # 添加获取锁，确保API获取是单线程的
        self.fetch_lock = threading.Lock()
        # 添加is_fetching锁
        self.fetching_lock = threading.Lock()
        # 添加stats锁
        self.stats_lock = threading.Lock()
        # 标记是否正在从API获取代理
        self.is_fetching = False
        self.stats = {
            '总数量': 0,  # 所有代理IP的总数量
            '可使用数量': 0,  # 可以立即使用的有效代理IP数量
            '等待验证': 0,  # 等待验证的代理数量
            '验证失败': 0,  # 验证失败的代理数量
            '正在验证': 0,  # 正在验证中的代理数量
            '已取数量': 0,  # 已从池中取走的代理数量
            'api获取数量': 0,  # 从API添加到池中的代理数量
            '失效数量': 0  # 失效的代理IP数量

        }

    def get_one_proxy(self, timeout=None):  # 获取一个代理IP，支持超时参数
        # 立即尝试获取
        self.check_proxy_list_num_async()
        proxy = self._get_proxy_from_pool()
        if proxy:
            # 使用stats锁保护统计更新
            with self.stats_lock:
                self.stats['已取数量'] += 1
                self.stats['可使用数量'] -= 1
            return proxy

        # 无限等待模式
        if timeout == -1:
            while not (proxy := self._get_proxy_from_pool()):
                time.sleep(0.1)
            # 使用stats锁保护统计更新
            with self.stats_lock:
                self.stats['已取数量'] += 1
                self.stats['可使用数量'] -= 1
            return proxy

        # 有限超时或无超时模式
        start_time = time.time()
        while timeout is not None and time.time() - start_time < timeout:
            if proxy := self._get_proxy_from_pool():
                # 使用stats锁保护统计更新
                with self.stats_lock:
                    self.stats['已取数量'] += 1
                    self.stats['可使用数量'] -= 1
                return proxy
            time.sleep(0.1)

        return None

    def _check_proxy_threshold(self):
        pool_size=0
        """检查代理池数量是否低于阈值，如是则启动获取"""
        if self.redis_client and self.redis_prefix:
            # Redis模式
            pool_key = f"{self.redis_prefix}:proxy_pool"
            pool_size = self.redis_client.scard(pool_key)
        else:
            # 内存模式
            with self.proxy_pool_lock:
                pool_size = len(self.proxy_pool)
        if pool_size < self.proxy_config.threshold:

            # 检查是否已经在获取中
            if not self.is_fetching:
                self.check_proxy_list_num_async()

    def _get_proxy_from_pool(self):
        self.check_proxy_list_num_async()
        """从代理池中获取一个代理"""
        if self.redis_client and self.redis_prefix:
            pool_key = f"{self.redis_prefix}:proxy_pool"
            # 使用spop直接弹出一个代理，更高效
            proxy_bytes = self.redis_client.spop(pool_key)
            if proxy_bytes:
                proxy = proxy_bytes.decode('utf-8')
                proxy_key = f"{self.redis_prefix}:proxy:{proxy}"
                # 检查代理键是否仍然存在（未过期）
                if self.redis_client.exists(proxy_key):
                    return proxy
            return None
        else:
            # 内存模式
            with self.proxy_pool_lock:
                if self.proxy_pool:
                    proxy = self.proxy_pool.pop(0)
                    return proxy
            return None

    def add_proxy(self, proxy):
        """
        添加代理IP到代理池
        :param proxy: 代理IP
        :return: None
        """
        if self.redis_client and self.redis_prefix:
            # Redis模式 - 使用独立键存储每个代理IP
            proxy_key = f"{self.redis_prefix}:proxy:{proxy}"
            pool_key = f"{self.redis_prefix}:proxy_pool"

            # 设置代理IP的值，并设置独立TTL
            self.redis_client.set(proxy_key, proxy)
            self.redis_client.expire(proxy_key, self.ttl)

            # 将代理IP添加到池集合中（用于快速查询）
            self.redis_client.sadd(pool_key, proxy)
            self.redis_client.expire(pool_key, self.ttl)  # 集合作为备份，设置较长TTL
            with self.stats_lock:
                self.stats['总数量'] += 1
                self.stats['可使用数量'] += 1


            
        else:
            # 内存模式
            with self.proxy_pool_lock:
                if proxy not in self.proxy_pool:
                    self.proxy_pool.append(proxy)
                    with self.stats_lock:
                        self.stats['总数量'] += 1
                        self.stats['可使用数量'] += 1
                    

    def delete_proxy(self, proxy):
        """
        从代理池中删除代理IP
        :param proxy: 代理IP
        :return: None
        """
        if self.redis_client and self.redis_prefix:
            # Redis模式
            proxy_key = f"{self.redis_prefix}:proxy:{proxy}"
            pool_key = f"{self.redis_prefix}:proxy_pool"

            # 删除代理IP的独立键
            self.redis_client.delete(proxy_key)
            # 从池集合中移除代理IP
            self.redis_client.srem(pool_key, proxy)
            with self.stats_lock:
                # 更新统计信息
                self.stats['总数量'] -= 1
                # 重新计算可用数量
                self.stats['可使用数量'] = self.redis_client.scard(pool_key)


        else:
            # 内存模式
            with self.proxy_pool_lock:
                if proxy in self.proxy_pool:
                    self.proxy_pool.remove(proxy)
                    with self.stats_lock:
                        self.stats['总数量'] -= 1
                        self.stats['可使用数量'] = len(self.proxy_pool)



    def clear_proxy(self):
        """
        清空代理池
        :return: None
        """
        if self.redis_client and self.redis_prefix:
            # Redis模式
            pool_key = f"{self.redis_prefix}:proxy_pool"
            # 获取池中所有代理IP
            all_proxies = self.redis_client.smembers(pool_key)
            # 删除所有代理IP的独立键
            for proxy_bytes in all_proxies:
                proxy = proxy_bytes.decode('utf-8')
                proxy_key = f"{self.redis_prefix}:proxy:{proxy}"
                self.redis_client.delete(proxy_key)
            # 删除池集合
            with self.proxy_pool_lock:
                self.redis_client.delete(pool_key)

                # 重置统计信息
                self.stats['总数量'] = 0
                self.stats['可使用数量'] = 0

        else:
            # 内存模式
            with self.proxy_pool_lock:
                self.proxy_pool.clear()
                self.stats['总数量'] = 0
                self.stats['可使用数量'] = 0

    def _verify_proxy(self, proxy):

        """验证代理IP的可用性"""
        try:
            self.logger.info(f"正在验证代理IP {proxy}")
            with self.stats_lock:
                self.stats['等待验证'] -= 1
                self.stats['正在验证'] += 1

            proxies = {
                'http': f'http://{proxy}',
                'https': f'http://{proxy}'
            }
            response = requests.get(
                self.proxy_config.check_url,
                proxies=proxies,
                timeout=self.proxy_config.timeout
            )
            self.logger.info(f"验证代理IP {proxy} 成功")
            with self.stats_lock:
                self.stats['正在验证'] -= 1
            
            return response.status_code == 200
        except Exception:
            self.logger.error(f"验证代理IP {proxy} 失败")
            with self.stats_lock:
                self.stats['正在验证'] -= 1
            stats = self.get_proxy_stats()
            self.logger.info(stats)
            return False

    def get_proxy_stats(self):  # 获取代理IP统计信息例如 可用的代理IP数量 等待验证的IP数量 验证失败的IP数量 验证中的IP数量 总代理IP数量 已提取的代理IP数量 从API获取的代理IP数量
        return self.stats

    def check_proxy_list_num_async(self):
        """异步检查池中数量是否小于阈值"""
        # 使用获取锁确保只有一个线程能启动获取任务
        if self.fetch_lock.acquire(blocking=False):
            try:
                # 使用fetching锁保护is_fetching标志
                with self.fetching_lock:
                    # 再次检查是否已经在获取中
                    if not self.is_fetching:
                        self.is_fetching = True
                        # 创建一个新线程来执行异步任务
                        thread = threading.Thread(target=self._async_fetch_proxies_if_needed)
                        thread.daemon = True
                        thread.start()
            finally:
                self.fetch_lock.release()

    def _async_fetch_proxies_if_needed(self):
        """异步获取代理IP（如果数量不足阈值）"""
        try:
            # 检查上次获取时间，确保满足间隔要求
            current_time = time.time()
            if hasattr(self, 'last_fetch_time'):
                time_since_last_fetch = current_time - self.last_fetch_time
                if time_since_last_fetch < self.proxy_config.get_time:
                    self.logger.info(f"等待 {self.proxy_config.get_time - time_since_last_fetch:.2f} 秒后再次获取")
                    # 等待到满足获取间隔
                    time.sleep(self.proxy_config.get_time - time_since_last_fetch)
            else:
                self.logger.info("首次获取代理IP")
            # 初始化检查变量
            need_fetch = False
            if self.redis_client and self.redis_prefix:
                # Redis模式
                pool_key = f"{self.redis_prefix}:proxy_pool"
                pool_size = self.redis_client.scard(pool_key)
                need_fetch = pool_size < self.proxy_config.threshold
            else:
                # 内存模式
                with self.proxy_pool_lock:
                    need_fetch = len(self.proxy_pool) < self.proxy_config.threshold

            if not need_fetch:
                return  # 数量足够，不需要获取新代理
            try:
                # 从API获取代理IP
                response = requests.get(self.proxy_config.api_url, timeout=self.proxy_config.timeout)
                if response.status_code == 200:
                    content = response.text
                    self.logger.info(f"从API获取代理IP成功{content}")
                    # 提取代理IP
                    proxies = re.findall(ip_pattern, content)

                    # 使用stats锁保护统计更新
                    with self.stats_lock:
                        self.stats['api获取数量'] += len(proxies)

                    if self.proxy_config.verify_proxy and proxies:
                        # 使用线程池验证代理IP
                        self._verify_proxies_with_thread_pool(proxies)
                        self.last_fetch_time = time.time()
            except Exception as e:
                self.logger.error(f"异步获取代理时出错: {e}")
        finally:
            # 使用fetching锁保护is_fetching标志
            with self.fetching_lock:
                # 重置获取状态
                self.is_fetching = False

    def _verify_proxies_with_thread_pool(self, proxies):
        """
        使用线程池验证代理IP

        Args:
            proxies: 代理IP列表
        Returns:
            list: 验证通过的代理IP列表
        """
        # 使用线程池执行器进行验证
        with self.stats_lock:
            self.stats['等待验证'] = len(proxies)

        with concurrent.futures.ThreadPoolExecutor(max_workers=self.proxy_config.verify_thread_num) as executor:
            # 提交所有验证任务
            future_to_proxy = {executor.submit(self._verify_proxy, proxy): proxy for proxy in proxies}

            valid_proxies = []
            for future in concurrent.futures.as_completed(future_to_proxy):
                proxy = future_to_proxy[future]
                try:
                    is_valid = future.result()
                    self.logger.info(f"验证代理IP {proxy} {is_valid}")
                    if is_valid:

                        self.add_proxy(proxy)

                    else:
                        with self.stats_lock:
                            # 验证失败的代理数量增加
                            #self.stats['等待验证'] -= 1
                            self.stats['验证失败'] += 1

                except Exception as e:
                    with self.stats_lock:
                        self.stats['验证失败'] += 1
                        #self.stats['等待验证'] -= 1

                    self.logger.error(f"验证代理 {proxy} 时出错: {e}")
                # finally:
                #     with self.stats_lock:
                #         # 减少等待验证的数量
                #         self.stats['等待验证'] -= 1


            return valid_proxies
if __name__ == '__main__':
    root_logger = logging.getLogger()
    root_logger.handlers.clear()

    logging.basicConfig(
        level=logging.DEBUG,
        format='%(asctime)s - %(levelname)s - %(filename)s:%(lineno)d - %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        handlers=[logging.StreamHandler(stream=sys.stdout)]  # 关键：指定输出到stdout
    )

    # 获取logger实例
    logger = logging.getLogger(__name__)
    proxy_config = ProxyConfig()
    proxy_config.set_threshold(10)
    proxy_config.use_redis=False
    proxy_config.set_get_time(3)
    proxy_class = ProxyClass(proxy_config,logger)
    proxy_class.check_proxy_list_num_async()
    for i in range(100000000000000):
        # 每0.1秒查询一次状态
        stats = proxy_class.get_proxy_stats()
        # logger.info(stats)

        # 每10秒获取一次IP
        proxy_one = proxy_class.get_one_proxy(-1)
        if proxy_one:
            # logger.info(f"获取的代理IP为: {proxy_one}")
            print(proxy_one)



        time.sleep(0.1)  # 每0.1秒循环一次


    print("等待演示")
    time.sleep(1000)
